;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# dataflash file manager 							#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
confp_main:	ldi	XL,1
		libmio_chpump			;enable chargepump for serial
		sts	libmio_channel,const_0	;output to screen
		sts	libmio_vidmode,const_0	;standard video mode
		ldi	XL,0x07			;color scheme (white on black)
		libmio_setcolor
		ldi	YL,0			;maximal sized box
		ldi	YH,0
		ldi	ZL,29
		ldi	ZH,22
		libmio_whbox
		ldi	XL,0x70			;color scheme (white on black)
		libmio_setcolor
		libmio_thistext
		.db 1,2,"AVR-ChipBasic2 configpage",0
		ldi	XL,0x07			;color scheme (white on black)
		libmio_setcolor
		libmio_thistext
		.db 20,1,"Checksum: ",0,0
		rcall	confp_chksum
		rjmp	confp_main_03

;------------------------------------------------------------------------------
;the main loop
;------------------------------------------------------------------------------
confp_main_02:	sts	libmio_sysconf,tempreg3		;recent conf
		sts	libmio_sysconf1,tempreg2	;recent conf
confp_main_03:	libmio_nokey
		rcall	confp_sconf			;view
		ldi	ZL,LOW(confp_mbar*2)
		ldi	ZH,HIGH(confp_mbar*2)
		clt				;read menu from flash
		libmio_menu
		lds	tempreg3,libmio_sysconf		;recent conf
		lds	tempreg2,libmio_sysconf1	;recent conf

;------------------------------------------------------------------------------
;esc key
;------------------------------------------------------------------------------
confp_main_30:	cpi	tempreg1,0x01		;esc
		brne	confp_main_50
		call	mem_setconf
		jmp	0			;restart

;------------------------------------------------------------------------------
;F1 key (change keyboard layout)
;------------------------------------------------------------------------------
confp_main_50:	cpi	tempreg1,0x02		;F1
		brne	confp_main_55
		ldi	tempreg4,0x20		;change bit 5
		eor	tempreg3,tempreg4
		rjmp	confp_main_02

;------------------------------------------------------------------------------
;F2 key (change serial speed)
;------------------------------------------------------------------------------
confp_main_55:	cpi	tempreg1,0x03		;F2
		brne	confp_main_60
		mov	tempreg4,tempreg3
		inc	tempreg4		;baud +1
		andi	tempreg4,0x01
		andi	tempreg3,0xf8
		or	tempreg3,tempreg4
		rjmp	confp_main_02

;------------------------------------------------------------------------------
;F3 key (change I2C speed)
;------------------------------------------------------------------------------
confp_main_60:	cpi	tempreg1,0x04		;F3
		brne	confp_main_65
		lds	tempreg1,libmio_sysconf	;recent conf
		ldi	tempreg4,0x10
		eor	tempreg3,tempreg4
		rjmp	confp_main_02

;------------------------------------------------------------------------------
;F4 key (change SPI speed)
;------------------------------------------------------------------------------
confp_main_65:	cpi	tempreg1,0x05		;F4
		brne	confp_as_01
		ldi	tempreg4,0x08
		eor	tempreg3,tempreg4
confp_main_67:	rjmp	confp_main_02

;------------------------------------------------------------------------------
;CTRL+ESC key (change Autostart)
;------------------------------------------------------------------------------
confp_as_01:	cpi	tempreg1,0x06		;F4
		brne	confp_main_70
		mov	tempreg4,tempreg2
		andi	tempreg2,0x8f
		subi	tempreg4,0xf0
		andi	tempreg4,0x70
		or	tempreg2,tempreg4
confp_as_02:	rjmp	confp_main_02

;------------------------------------------------------------------------------
;C+F1 key (change Data EEPROM address)
;------------------------------------------------------------------------------
confp_main_70:	cpi	tempreg1,0x07		;C+F1
		brne	confp_main_75
		mov	tempreg4,tempreg2
		inc	tempreg4
		andi	tempreg4,0x07
		andi	tempreg2,0xf8
		or	tempreg2,tempreg4
		rjmp	confp_main_02

;------------------------------------------------------------------------------
;C+F2 Line end
;------------------------------------------------------------------------------
confp_main_75:	cpi	tempreg1,0x08		;C+F2
		brne	confp_main_77
		subi	tempreg3,0xc0		;+0x40
		brcs	confp_main_76
		ori	tempreg3,0x40
confp_main_76:	rjmp	confp_main_02

;------------------------------------------------------------------------------
;C+F3 Serin
;------------------------------------------------------------------------------
confp_main_77:	cpi	tempreg1,0x09		;C+F3
		brne	confp_main_80
		subi	tempreg2,0x80		;+0x80
		rjmp	confp_main_02

;------------------------------------------------------------------------------
;ctrl+F4 clone
;------------------------------------------------------------------------------
confp_main_80:	cpi	tempreg1,0x0a		;ctrl-F4
		brne	confp_main_90
		call	clone_main
confp_main_90:	rjmp	confp_main


confp_sconf:	;show config
;------------------------------------------------------------------------------
;show keyboard layout
;------------------------------------------------------------------------------
confp_kbdtype: 	ldi	XL,0x06			;color scheme (yellow on black)
		libmio_setcolor
		libmio_thistext
		.db 3,1,"Keyboard layout:",0,0
		libmio_thistext
		.db 5,1,"Serial speed:",0
		libmio_thistext
		.db 7,1,"I2C speed:",0,0
		libmio_thistext
		.db 9,1,"SPI speed:",0,0
		libmio_thistext
		.db 11,1,"EEPROM address:",0
		libmio_thistext
		.db 13,1,"Line end:",0
		libmio_thistext
		.db 15,1,"Serial interface:",0
		libmio_thistext
		.db 17,1,"Autostart:",0,0
		ldi	XL,0x71			;blue on white
		libmio_setcolor
		lds	tempreg4,libmio_sysconf	;get active layout
		sbrc	tempreg4,5
		rjmp	confp_kbdtype1
		libmio_thistext
		.db 3,18,"DE (QWERTZ)",0
		rjmp	confp_kbdtype2
confp_kbdtype1:	libmio_thistext
		.db 3,18,"US (QWERTY)",0
confp_kbdtype2:

;------------------------------------------------------------------------------
;show serial speed
;------------------------------------------------------------------------------
		sbrc	tempreg4,0
		rjmp	confp_sertype1
		libmio_thistext
		.db 5,18," 1200 Bps  ",0
		rjmp	confp_sertype2
confp_sertype1:	libmio_thistext
		.db 5,18," 2400 Bps  ",0
confp_sertype2:

;------------------------------------------------------------------------------
;show I2C speed
;------------------------------------------------------------------------------
		sbrc	tempreg4,4
		rjmp	confp_twitype1
		libmio_thistext
		.db 7,18,"  100 kHz  ",0
		rjmp	confp_twitype2
confp_twitype1:	libmio_thistext
		.db 7,18,"  400 kHz  ",0
confp_twitype2:


;------------------------------------------------------------------------------
;show SPI speed
;------------------------------------------------------------------------------
		sbrc	tempreg4,3
		rjmp	confp_spitype1
		libmio_thistext
		.db 9,18,"    5 MHz  ",0
		rjmp	confp_spitype2
confp_spitype1:	libmio_thistext
		.db 9,18,"  156 kHz  ",0
confp_spitype2:

;------------------------------------------------------------------------------
;show line end
;------------------------------------------------------------------------------
		andi	tempreg4,0xc0
confp_lintype1:	cpi	tempreg4,0x80
		brne	confp_lintype2
		libmio_thistext
		.db 13,18," only LF   ",0
		rjmp	confp_lintype9
confp_lintype2:	cpi	tempreg4,0x40
		brne	confp_lintype3
		libmio_thistext
		.db 13,18," only CR   ",0
		rjmp	confp_lintype9
confp_lintype3:	libmio_thistext
		.db 13,18,"  CR+LF    ",0
confp_lintype9:

;------------------------------------------------------------------------------
;show serial interface
;------------------------------------------------------------------------------
		lds	tempreg4,libmio_sysconf1
		sbrc	tempreg4,7
		rjmp	confp_sintype1
		libmio_thistext
		.db 15,18,"  simple   ",0
		rjmp	confp_sintype2
confp_sintype1:	libmio_thistext
		.db 15,18," standard  ",0
confp_sintype2:

;------------------------------------------------------------------------------
;show EEPROM address
;------------------------------------------------------------------------------
confp_eeadr:	libmio_thistext
		.db 11,18,"     ",0
		mov	tempreg1,tempreg4
		andi	tempreg1,0x07
		subi	tempreg1,0xd0
		libmio_outchar
	 	libmio_thistext
		.db 255,"     ",0,0

;------------------------------------------------------------------------------
;show autostart
;------------------------------------------------------------------------------
confp_autos:	libmio_thistext
		.db 17,18," PRG ",0
		mov	tempreg1,tempreg4
		swap	tempreg1
		andi	tempreg1,0x07
		breq	confp_autos1
		subi	tempreg1,0xd0
		libmio_outchar
		libmio_thistext
		.db 255,"     ",0,0
confp_autose:	ldi	XL,0x07			;color scheme (white on black)
		libmio_setcolor
		ret

confp_autos1:	libmio_thistext
		.db 255,"off   ",0
		rjmp	confp_autose


;------------------------------------------------------------------------------
;calc and output checksum
;------------------------------------------------------------------------------
confp_chksum:	clr	XL			;sum
		clr	YL
		clr	YH
		clr	ZL			;ptr
		clr	ZH

confp_chksum1:	lpm	r1,Z+
		add	YL,r1	
		adc	YH,const_0
		adc	XL,const_0
		cpi	ZH,HIGH(end_system*2)
		brne	confp_chksum1
		clr	ZL			;ptr
		ldi	ZH,$f0			;

confp_chksum2:	lpm	r1,Z+
		add	YL,r1
		adc	YH,const_0
		adc	XL,const_0
		cpi	ZH,0x00
		brne	confp_chksum2
		clr	XH
		ldi	ctrl,0x0f
		libmio_outhex
		movw	XL,YL
		libmio_outhex
		ret

;------------------------------------------------------------------------------
;the configpage menubar
;------------------------------------------------------------------------------
		;   "111111222222333333444444555555"
confp_mbar:	.db "Reboot Keyb  Ser   I2C   SPI  "
		.db "AutostEEPROM Line Serint Clone"
